/*
 * Sketch for a counter display that shows live
 * how many play clicks a certain user has had
 * on SoundCloud so far. 
 * 
 * 
 */

#include <ArduinoJson.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>

WiFiClient client;

char ssid[] = "Residenz";       // your network SSID (name)
char password[] = "alteburg";   // your network key
// char SCUserID = "229446377";  //unique SoundCloud user ID (just a long number)
// char SCApiClientID = "JlZIsxg2hY5WnBgtn3jfS0UYCl0K8DOg"; //your own API key

const char* server = "mixedtinkerings.com";   // server's address
const char* resource = "/tillefon/calcscplays.php?SCUserID=229446377&SCApiClientID=JlZIsxg2hY5WnBgtn3jfS0UYCl0K8DOg";  //http resource to parse 
const unsigned long BAUD_RATE = 115200;      // serial connection speed
const unsigned long HTTP_TIMEOUT = 10000;    // max respone time from server
const size_t MAX_CONTENT_SIZE = 45;        // max size of the HTTP response 

// The type of data that we want to extract from the page
struct UserData {
  long playCount;
};

long totalPlays = 0;
long prevPlays = 0;

int delayInSecs = 10;      // time for wait() function in seconds
int showDelay = 100;       // time in ms between number dance sequence

//Pin connected to latchpin
int latchPin = D1;
//Pin connected to clockpin
int clockPin = D2;
////Pin connected to datapin
int dataPin = D0;

// repository for all glyphs of the nixies incl "all glyphs off"-glyph
char* glyphs[11] = {
  "0000000001",       // Nixie digit "0" on first pin of register
  "1000000000",       // Nixie digit "1" on tenth pin of register
  "0100000000", 
  "0010000000", 
  "0001000000", 
  "0000100000",
  "0000010000",
  "0000001000",
  "0000000100",
  "0000000010",
  "0000000000"
};


// ARDUINO entry point #1: runs once when you press reset or power the board
void setup() {
  initSerial();
  initInternet();
  //set pins to output because they are addressed in the main loop
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  digitalWrite(latchPin, HIGH);                  // shift register off
  digitalWrite(dataPin, HIGH);
  digitalWrite(clockPin, HIGH);
}

// ARDUINO entry point #2: runs over and over again forever
void loop() {
  if (connect(server)) {
   if (sendRequest(server, resource) && skipResponseHeaders()) {
     UserData userData;
     if (readResponseContent(&userData)) {
      printUserData(&userData);
     }
   }
   disconnect();
  }
  
  Serial.println(totalPlays);
  Serial.println(valueToGlyphs(totalPlays, 1));             // for debugging
  
  if (totalPlays != prevPlays) {
    bangOutGlyphstring(valueToGlyphs(0, 1));                // blank display
    delay(showDelay);
    for (int i=0; i<10; i++) {                              // prevent cathode poisoning
      String testString = valueToGlyphs((1111111 * i), 0);  
      bangOutGlyphstring(testString);                       
      delay(showDelay);
    }
    bangOutGlyphstring(valueToGlyphs(0, 1));                // blank display
    delay(showDelay);
    String megaString = valueToGlyphs(totalPlays, 1);       // make the string to send
    bangOutGlyphstring(megaString);                         // actually send the String 
    prevPlays = totalPlays; 
  } 
  wait();
}


// convert our value of playcount into string 
// containing sequences of 0s and 1s
// according to our glyp repository
String valueToGlyphs (long value, bool doKillZeroes) {
  String result = "";                                   // String object to hold final sequence
  long prevDecimal = 0;
  long compare = 1000000;                               // because we have 7 Nixies this is biggest divider
  while (compare >= 1) {
    long currentDecimal = value / compare;              // single out one digit of our play value
    if ((currentDecimal == 0) && (prevDecimal == 0) && doKillZeroes) {      // if digit before that was a zero, then:
      result = result + glyphs[10];                     // output "off-glyph"
    } else {                                            // if digit before that was not a zero, then
      result = result + glyphs[currentDecimal];           // add glyph representation of digit to String
      prevDecimal = 1;                                // set prevDecimal so that the next zero will also be output
    }
    value = value - (currentDecimal * compare);         // prepare value to get next digit
    compare = compare / 10;                             // adjust divider for next digit
  }
  
  return result;
}

// actually send String out to the shift registers
void bangOutGlyphstring (String glyphstring) {
  digitalWrite(latchPin, HIGH);                  // shift register off
  for(int i=0; i<glyphstring.length(); i++) {
    digitalWrite(clockPin, HIGH);                // LOW means don't read for the registers
    delay(1);                                    // !!! swap all HIGH and LOW b'cause of 74HCT04
    if (glyphstring[i] == '0') {                // check String wether to send a 0 or a 1
      digitalWrite(dataPin, HIGH);               // set data pin accordingly
      delay(1);
      //Serial.println("_");                      // Serial Monitor debugging
    } else {
      digitalWrite(dataPin, LOW);
      delay(1);
      //Serial.println("1");
    }
    digitalWrite(clockPin, LOW);               // when clockPin goes high the registers read the datapin
    delay(1);
  }
  digitalWrite(latchPin, LOW);                 // shift register on again
}

// Initialize Serial port
void initSerial() {
  Serial.begin(BAUD_RATE);
  while (!Serial) {
    ;  // wait for serial port to initialize
  }
  Serial.println("Serial ready");
}

// Initialize Internet library
void initInternet() {
  // Set WiFi to station mode and disconnect from an AP if it was Previously
  // connected
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);

  // Attempt to connect to Wifi network:
  Serial.print("\nConnecting Wifi: ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  IPAddress ip = WiFi.localIP();
  Serial.println(ip);
  delay(1000);
}

// Open connection to the HTTP server
bool connect(const char* hostName) {
  Serial.print("Connect to ");
  Serial.println(hostName);

  bool ok = client.connect(hostName, 80);

  Serial.println(ok ? "Connected" : "Connection Failed!");
  return ok;
}

// Send the HTTP GET request to the server
bool sendRequest(const char* host, const char* resource) {
  Serial.print("GET ");
  Serial.println(resource);

  client.print("GET ");
  client.print(resource);
  client.println(" HTTP/1.0");
  client.print("Host: ");
  client.println(host);
  client.println("Connection: close");
  client.println();

  return true;
}

// Skip HTTP headers so that we are at the beginning of the response's body
bool skipResponseHeaders() {
  // HTTP headers end with an empty line
  char endOfHeaders[] = "\r\n\r\n";

  client.setTimeout(HTTP_TIMEOUT);
  bool ok = client.find(endOfHeaders);

  if (!ok) {
    Serial.println("No response or invalid response!");
  }

  return ok;
}

// Parse the JSON from the input string and extract the interesting values
bool readResponseContent(struct UserData* userData) {
  // Compute optimal size of the JSON buffer according to what we need to parse.
  // See https://bblanchon.github.io/ArduinoJson/assistant/

  const size_t BUFFER_SIZE = 
    JSON_OBJECT_SIZE(1)         // only 1 item in JSON object
    + MAX_CONTENT_SIZE;          // additional space for strings as given at the top

  // Allocate a temporary memory pool
  DynamicJsonBuffer jsonBuffer(BUFFER_SIZE);
  Serial.println("JSON Buffer is: ");
  Serial.println(BUFFER_SIZE);

  JsonObject& root = jsonBuffer.parseObject(client);   //client response being parsed into root
  totalPlays = root["total_plays"];
  
  if (!root.success()) {
    Serial.println("JSON parsing failed!");
    return false;
  }

  // Here were copy the value we're interested in
  userData->playCount = totalPlays;

  return true;
}

// Print the data extracted from the JSON
void printUserData(const struct UserData* userData) {
  Serial.print("Total Play Count = ");
  Serial.println(userData->playCount);
}

// Close the connection with the HTTP server
void disconnect() {
  Serial.println("Disconnect");
  client.stop();
}


// Pause 
void wait() {
  Serial.println(" ");
  Serial.print("Wait ");
  Serial.print(delayInSecs);
  Serial.print(" seconds");
  Serial.println(" ");
  delay(delayInSecs * 1000);
}


